/* Schleifenbauer data base tables and triggers, DM v1.24

        Updates v1.44 to v1.46
        adds gw 2.46 and fw 1.46 support; input names, remote reset option, updated field descriptions
		exclude zero values from average in aggregation
*/


/* adjust data base name ! */
USE `sppdu`;


INSERT INTO dbhistory(version) VALUES("1.46");

DELETE FROM spdm WHERE `mnemonic` LIKE 'cfnrte';
DELETE FROM spdm WHERE `mnemonic` LIKE 'idspdm';
DELETE FROM spdm WHERE `mnemonic` LIKE 'idfwvs';


INSERT INTO `spdm` (`interval`, `mnemonic`, `name`, `register`, `description`,`group` ) VALUES
(0,  'idspdm', 'SPDMVersion', 100, 'Data model version (1.24 current). A register and its channels can have one or more extensions accessible via a new read and write command in the api (2 and 17) or adding 10.000 for Modbus. Outlets now have one extra set of registers, identical to the originals, but the channels are 28..48 instead of 1..27. ', 'identification' ),
(0,  'idfwvs', 'firmwareVersion', 102, 'The units firmware revision number (1.46 at the time of writing)', 'identification' ),
(0,  'cfnrte', 'n.o.Temp.Sensors', 205, 'The configuration of the sensor port: 0 for none, 1 for digital only. 2 for analog (int and ext). The actual number of digital sensor is returned by cfnres.', 'configuration' ),
(0,  'stinnm', 'inputName', 1152, 'User configurable naming of the inputs or phases. Useful if channels are used to measure branches.', 'settings' ),
(0,  'strsal', 'localAlertReset', 1054, 'Allow alerts to be reset locally on pdu by pushing one of the buttons.', 'settings' )
;


DELIMITER $$$

DROP PROCEDURE IF EXISTS `sp_insert_val`$$$
CREATE PROCEDURE `sp_insert_val`(insint INT, itime DATETIME, a INT, raw blob, x int, t tinyblob, b int, l int, n int, r tinyblob)
DETERMINISTIC
BEGIN
  DECLARE i INT;
  DECLARE pos INT;
  DECLARE cnt INT;
  DECLARE intv_cnt INT;
  DECLARE chvalue FLOAT; 
  DECLARE bString tinyblob;
  DECLARE tString VARCHAR(20);
  DECLARE extn INT;
  DECLARE cntvalue INT;

 SET cntvalue = 1; -- use value for averaging

  SELECT COUNT(`spdm`.`interval`) FROM `spdm` WHERE
	`mnemonic` = r
    INTO intv_cnt;

   IF (intv_cnt = 1) THEN
     SELECT `spdm`.`interval` FROM `spdm` WHERE
	`mnemonic` = r 
     INTO insint;
   END IF;

  SET cnt = 0;

/* only add records if insint more than 0 seconds */
if (insint = 0) THEN
  DELETE FROM `splog` WHERE `pdu` = a AND `register` = r AND `extended` = x ;
ELSE
/*
	if there are any recent entries in splog,
	thus from last insint seconds, then do not insert new records
*/
  SELECT COUNT(`splog`.`id`) FROM `splog` WHERE
	`pdu` = a AND
	`register` = r AND
    `extended` = x AND
	(TIMESTAMPDIFF (SECOND,`time`,itime) < insint)
    INTO cnt;

END IF;

if (cnt = 0) THEN
SET extn = n;
IF (x = 1) THEN
 SET extn = n-27;
ELSE
  IF (n>27) THEN
    SET extn=27;
  END IF;
END IF;

  SET i = 0;
      WHILE i < extn do
        SET cntvalue = 1; 
      CASE

        WHEN t like "fd" THEN
		BEGIN
			SET chvalue = sp_get_fd(substr(raw, (b + (i * l))+1,l));
			if chvalue = 0 then
				SET cntvalue = 0; -- exclude from averaging
			END IF;
          	INSERT INTO `splog` (`pdu`,`register`,`channel`,`value`,`time`,`min_v`,`max_v`,`extended`,`nvalues`)
            	VALUES (a,r,(x * 27)+i+1, chvalue, itime, chvalue, chvalue,x,cntvalue);
		END;

        WHEN t like "int" THEN
		BEGIN
			SET chvalue = sp_get_int(substr(raw, (b + (i * l))+1,l),l);
			if chvalue = 0 then
				SET cntvalue = 0; -- exclude from averaging
			END IF;
         	INSERT INTO `splog` (`pdu`,`register`,`channel`,`value`,`time`,`min_v`,`max_v`,`extended`,`nvalues`)
            	VALUES (a,r, (x * 27)+i+1, chvalue, itime, chvalue, chvalue,x,cntvalue);
		END;

        WHEN t like "ascii" THEN
		SET bString = substr(raw, (b + (i * l))+1,l);    /* get string in blob */
		SET pos = position("\0" IN bString);             /* find NUL terminator */

		IF (pos = 0) THEN
			SET pos = l+1;
		END IF;

		SET tString = substr(bString, 1, pos-1);         /* convert to C string */

         INSERT INTO `splog` (`pdu`,`register`,`channel`,`text`,`time`,`extended`)
            VALUES (a,r,(x * 27)+i+1, tString, itime,x);

        END CASE;

          SET i = i + 1;
      END WHILE;
	END IF;


END $$$


DROP TRIGGER IF EXISTS `sp_process_raw`$$$
CREATE TRIGGER `sp_process_raw`
BEFORE INSERT on `spregister`
FOR EACH ROW
BEGIN
DECLARE reg INT;
DECLARE cfnrph INT;
DECLARE cfnrno INT;
DECLARE cfnrso INT;
DECLARE cfnrmo INT;
DECLARE cfamps INT;
DECLARE cfnrte INT;
DECLARE cfnres INT;
SET reg = new.register;
SET new.inserttime=CURRENT_TIMESTAMP;


/* the first parameter is the minimal delay between two inserts.
0 will prevent insertion of records and instead update records,

use a zero setting for static data (configuration) and non zero for dynamic data (measurements)
*/

/*
	first check config records to see how many channels are configured for this register
	limit the number of inserted record to number of channels or even zero

	1.40 add extended attribute
*/

CASE reg
WHEN 100  THEN
BEGIN
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 2, 1, 'idspdm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 2, 1, 'idfwvs');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 4, 16, 1, 'idonbr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 20, 16, 1, 'idpart');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 36, 16, 1, 'idsnbr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 52, 2, 3, 'idchip');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 58, 2, 1, 'idaddr');
END;
WHEN 200  THEN
BEGIN
-- note: cfnr** must be static (interval 0)!
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, 1, 'cfnrph');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 1, 1, 1, 'cfnrno');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 1, 1, 'cfnrso');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 3, 1, 1, 'cfnrmo');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 4, 1, 1, 'cfamps');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 5, 1, 1, 'cfnrte');
                    CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 6, 1, 1, 'cfnres');
END; 
WHEN 300  THEN
BEGIN
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, 1, 'ssstat');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 1, 1, 1, 'ssttri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 1, 1, 'ssitri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 3, 1, 1, 'ssotri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 4, 1, 1, 'ssvtri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 5, 1, 1, 'ssftri');
                    CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 6, 1, 1, 'ssicda');
END;
WHEN 400  THEN 
BEGIN
END; 
WHEN 1000  THEN 
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrph' INTO cfnrph;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrmo' INTO cfnrmo;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrno' INTO cfnrno;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrso' INTO cfnrso;
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 0, 16, 1, 'stdvnm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 16, 16, 1, 'stdvlc');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 32, 20, 1, 'stuser');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 52, 2, 1, 'stpkdr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 54, 1, 1, 'strsal');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 56, 2, 1, 'stfodl');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 58, 1, 1, 'stpsav');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 59, 1, 1, 'stopom');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 60, 1, 1, 'stmaxt');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 61, 1, 1, 'stdiso');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'fd', 62, 2, cfnrph, 'stimcm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'fd', 68, 2, cfnrmo, 'stomcm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 122, 1, cfnrno, 'stomct');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 149, 1, cfnrph, 'stimct');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii',152, 8, cfnrph, 'stinnm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 176, 8, cfnrno, 'stolnm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 392, 2, cfnrso, 'stiodl');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 446, 1, 1, 'stcddt');
END;
WHEN 2000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrso' INTO cfnrso;
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, cfnrso, 'swocst');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 27, 1, cfnrso, 'swosch');
END;
WHEN 3000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrph' INTO cfnrph;
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 0, 3, cfnrph, 'imkwht');
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 9, 3, cfnrph, 'imkwhs');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 18, 2, cfnrph, 'impfac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 24, 2, cfnrph, 'imcrac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 30, 2, cfnrph, 'imcrpk');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 36, 2, cfnrph, 'imvoac');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 42, 2, cfnrph, 'imvodp');
END;
WHEN 4000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrmo' INTO cfnrmo;
 	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int',  0, 3, cfnrmo, 'omkwht');
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 81, 3, cfnrmo, 'omkwhs');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 162, 2, cfnrmo, 'ompfac');
 	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 216, 2, cfnrmo, 'omcrac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 270, 2, cfnrmo, 'omcrpk');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 324, 2, cfnrmo, 'omvoac');
END;
WHEN 5000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrte' INTO cfnrte;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnres' INTO cfnres;
	IF (cfnrte>0) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 0, 2, 1, 'pditem');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 4, 2, 1, 'pdinpk');
	END IF;
	IF (cfnrte>1) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 2, 2, 1, 'pdetem');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 6, 2, 1, 'pdexpk');
	END IF;
	IF (cfnres>1) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'ascii', 8, 1, cfnres, 'snstyp');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd',   24, 2, cfnres, 'snsval');
        CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'ascii', 56, 6, cfnres, 'snsnme');
	END IF;
END;
 ELSE
  BEGIN
  END;

END CASE;


SET new.done = 1;

END $$$


/*
-- add aggregation function, averages values over a time range and deletes records

1.30 limit to non-accumulating measurements
1.46 exception for division by zero
*/
DROP PROCEDURE IF EXISTS `sp_aggregate_values`$$$

CREATE PROCEDURE `sp_aggregate_values`(IN AGGR TINYINT, IN CUTDT DATETIME)
DETERMINISTIC
    COMMENT 'Aggregates using MySql BCD-like representation of timestamps'
BEGIN
  declare AGGRT int; declare MAXDT datetime; declare MINA double;
  set AGGRT = pow(10,AGGR);
  set MINA = case when AGGR = 8 then 0.01 else 0 end;
  set MAXDT = cast((floor(CUTDT/AGGRT)+MINA)*AGGRT as datetime);
  replace splog (`id`,`pdu`,`register`,`channel`,`time`,`value`,`min_v`,`max_v`,`nvalues`,`aggregate`)
  select
  max(id) as id,
  pdu, register, channel,
  max(time) as time,
  cast(sum(value*nvalues)/IF(sum(nvalues),sum(nvalues),1)  as decimal(9,1)) as value, -- weighted avg, exception for division by zero
  min(min_v) as min_v,
  max(max_v) as max_v,
  sum(nvalues) as nvalues,
  AGGR as aggregate
  from splog e
  where e.time < MAXDT and e.aggregate < AGGR and 
	(
		LEFT(e.register,3) = "imp" OR
		LEFT(e.register,3) = "imc" OR
		LEFT(e.register,3) = "imv" OR
		LEFT(e.register,3) = "omp" OR
		LEFT(e.register,3) = "omc" OR
		LEFT(e.register,3) = "omv" OR
		LEFT(e.register,2) = "pd"
	)
  group by pdu,register,channel,floor(e.time/AGGRT)
  ;

  delete from splog where `time` < MAXDT and `aggregate` < AGGR AND
	(
		LEFT(register,3) = "imp" OR
		LEFT(register,3) = "imc" OR
		LEFT(register,3) = "imv" OR
		LEFT(register,3) = "omp" OR
		LEFT(register,3) = "omc" OR
		LEFT(register,3) = "omv" OR
		LEFT(register,2) = "pd" 
	);

END
$$$
